#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pthread.h>

#include "avr-isp.h"
#include "gui.h"

void
on_buttonReadFuse_clicked (GtkButton * button, gpointer user_data)
{
   unsigned char fuses[3];
   int i;
   char buf[128];
   GtkWidget *tmp;

   /* Just to make sure correct options are shown */
   on_buttonAVRID_clicked (button, user_data);

   /* Read fuse and lock bytes */
   if (avr_readfuses (fuses))
   {
      show_error ("Could not read fuse values\n");
      return;
   }
   for (i = 0; i < 8; i++)
   {
      sprintf (buf, "%s%d", "checkbuttonFuseL", i);
      tmp = lookup_widget (mainwindow, buf);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp),
                                    (gboolean) ! (fuses[0] & (1 << i)));
      sprintf (buf, "%s%d", "checkbuttonFuseH", i);
      tmp = lookup_widget (mainwindow, buf);
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp),
                                    (gboolean) ! (fuses[1] & (1 << i)));
   }
}

void
on_quit1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
   exit (0);
}


void
on_buttonBrowseHEX_clicked (GtkButton * button, gpointer user_data)
{
   GtkWidget *tmp;
   GtkWidget *dialog;
   GtkFileFilter *filterhex, *filterall;

   dialog = gtk_file_chooser_dialog_new ("Open File",
                                         GTK_WINDOW ((GTK_WIDGET
                                                      (button))->parent->
                                                     parent),
                                         GTK_FILE_CHOOSER_ACTION_OPEN,
                                         GTK_STOCK_CANCEL,
                                         GTK_RESPONSE_CANCEL,
                                         GTK_STOCK_OPEN,
                                         GTK_RESPONSE_ACCEPT, NULL);
   filterhex = gtk_file_filter_new ();
   gtk_file_filter_set_name (filterhex, "HEX files");
   gtk_file_filter_add_pattern (filterhex, "*.hex");
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filterhex);
   filterall = gtk_file_filter_new ();
   gtk_file_filter_set_name (filterall, "all files");
   gtk_file_filter_add_pattern (filterall, "*");
   gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (dialog), filterall);

   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
   {
      char *filename;

      filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      /* If called by EEPROM-filechooser, then call with -1 */
      if ((int) user_data == -1)
      {
         tmp = lookup_widget (gtk_widget_get_parent
                              (GTK_WIDGET (button)), "entryEEPROMHEX");
      }
      else
      {
         tmp = lookup_widget (gtk_widget_get_parent
                              (GTK_WIDGET (button)), "entryFlashHEX");
      }
      if (tmp != NULL)
      {
         gtk_entry_set_text (GTK_ENTRY (tmp), filename);
      }
   }

   gtk_widget_destroy (dialog);
}

void
on_togglebuttonEndian_toggled (GtkToggleButton * togglebutton,
                               gpointer user_data)
{
   FLAG_ENDIAN = (gtk_toggle_button_get_active (togglebutton) == TRUE);
}

void
on_buttonAVRID_clicked (GtkButton * button, gpointer user_data)
{
   int ret;
   GtkWidget *tmp;

   /* Read ID-bytes from AVR and determine type */
   if ((ret = avr_readID (&this_device)))
   {
      show_error ("Could not sync or read ID from device.\n");
      return;
   }
   /* On succes change selected AVR in combo-box */
   tmp = lookup_widget (mainwindow, "comboboxDevice");
   if (tmp)
   {
      int i;

      for (i = 0; this_device != &(id_codes[i]); i++)
         ;
      gtk_combo_box_set_active (GTK_COMBO_BOX (tmp), i + 1);
   }
}

void
on_comboboxDevice_changed (GtkComboBox * combobox, gpointer user_data)
{
   int deviceind;
   GtkWidget *tmp;
   int i;
   char buf[128];

   deviceind = gtk_combo_box_get_active (combobox);
   this_device = &(id_codes[deviceind - 1]);
   if (deviceind >= 0)
   {
      if (id_codes[deviceind - 1].fusename_l != NULL)
      {
         for (i = 0; i < 8; i++)
         {
            sprintf (buf, "%s%d", "checkbuttonFuseL", i);
            tmp = lookup_widget (gtk_widget_get_parent
                                 (GTK_WIDGET (combobox)), buf);
            gtk_button_set_label (GTK_BUTTON (tmp),
                                  id_codes[deviceind - 1].fusename_l[7 - i]);
            gtk_widget_set_sensitive (tmp, TRUE);
         }
      }
      else
      {
         for (i = 0; i < 8; i++)
         {
            sprintf (buf, "%s%d", "checkbuttonFuseL", i);
            tmp = lookup_widget (gtk_widget_get_parent
                                 (GTK_WIDGET (combobox)), buf);
            gtk_button_set_label (GTK_BUTTON (tmp), "N/A");
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), FALSE);
            gtk_widget_set_sensitive (tmp, FALSE);
         }
      }
      if (id_codes[deviceind - 1].fusename_h != NULL)
      {
         for (i = 0; i < 8; i++)
         {
            sprintf (buf, "%s%d", "checkbuttonFuseH", i);
            tmp = lookup_widget (gtk_widget_get_parent
                                 (GTK_WIDGET (combobox)), buf);
            gtk_button_set_label (GTK_BUTTON (tmp),
                                  id_codes[deviceind - 1].fusename_h[7 - i]);
            gtk_widget_set_sensitive (tmp, TRUE);
         }
      }
      else
      {
         for (i = 0; i < 8; i++)
         {
            sprintf (buf, "%s%d", "checkbuttonFuseH", i);
            tmp = lookup_widget (gtk_widget_get_parent
                                 (GTK_WIDGET (combobox)), buf);
            gtk_button_set_label (GTK_BUTTON (tmp), "N/A");
            gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (tmp), FALSE);
            gtk_widget_set_sensitive (tmp, FALSE);
         }
      }
   }
}

void
on_buttonQuit_clicked (GtkButton * button, gpointer user_data)
{
   exit (0);
}

void
on_buttonBrowseEEPROMHEX_clicked (GtkButton * button, gpointer user_data)
{
   on_buttonBrowseHEX_clicked (button, (gpointer) - 1);
}

void
on_buttonWriteFuse_clicked (GtkButton * button, gpointer user_data)
{
   unsigned char fuses[3] = { 0, 0, 0 };
   int i;
   char buf[128];
   GtkWidget *tmp;
   struct ID_CODES *previous = this_device;

   /* Make sure that correct values are displayed */
   on_buttonAVRID_clicked (button, user_data);
   if (previous != this_device)
   {
      show_error ("Apparently you changed the AVR, check again!");
      return;
   }

   /* Construct values to be programmed */
   for (i = 0; i < 8; i++)
   {
      sprintf (buf, "%s%d", "checkbuttonFuseL", i);
      tmp = lookup_widget (mainwindow, buf);
      fuses[0] +=
         (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp))) << i;
      sprintf (buf, "%s%d", "checkbuttonFuseH", i);
      tmp = lookup_widget (mainwindow, buf);
      fuses[1] +=
         (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (tmp))) << i;
   }

   /* Write fuse */
   if (avr_writefuses (fuses))
   {
      show_error ("Could not write fuse values\n");
      return;
   }
}

void
on_about1_activate (GtkMenuItem * menuitem, gpointer user_data)
{
   GtkWidget *msgdlg;
   char      *buffer;
   int       i;
   
   for(i=0; id_codes[ i].ID; i++)
      ;
   buffer = malloc( strlen(ABOUTMSG) + i*24);
   strcpy( buffer, ABOUTMSG);
   for(i=0;id_codes[ i].ID; i++)
   {
      strcat( buffer, "    ");
      strcat( buffer, id_codes[ i].device);
      strcat( buffer, "\n");
   }

   msgdlg = gtk_message_dialog_new (GTK_WINDOW (mainwindow),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_INFO,
                                    GTK_BUTTONS_CLOSE, buffer);
   gtk_dialog_run (GTK_DIALOG (msgdlg));
   gtk_widget_destroy (msgdlg);
   free( buffer);
}

void
on_buttonProgram_clicked (GtkButton * button, gpointer user_data)
{
   FILE *fpflash = NULL;
   FILE *fpeeprom = NULL;
   GtkWidget *tmp, *progress;
   struct ID_CODES *previous = this_device;
   int ret;

   /* Make sure that correct AVR is displayed */
   on_buttonAVRID_clicked (button, user_data);
   if (previous != this_device)
   {
      show_error ("Apparently you changed the AVR, check again!");
      return;
   }
   /* Open HEX files */
   tmp = lookup_widget (mainwindow, "entryFlashHEX");
   fpflash = fopen (gtk_entry_get_text (GTK_ENTRY (tmp)), "r");
   tmp = lookup_widget (mainwindow, "entryEEPROMHEX");
   fpeeprom = fopen (gtk_entry_get_text (GTK_ENTRY (tmp)), "r");

   /* Show progres bar */
   progress = lookup_widget (mainwindow, "progressbarFLASH");
   gtk_widget_show (progress);
   gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progress), 0.0);

   if ((ret = avr_program (fpflash, fpeeprom, update_progress)))
   {
      if (ret == -1)
         show_error ("Error during FLASH programming!\n");
      else
         show_error ("Error during EEPROM programming!\n");
   }

   gtk_widget_hide (progress);
   
   if (fpflash != NULL)
   {
      fclose (fpflash);
   }
   if (fpeeprom != NULL)
   {
      fclose (fpeeprom);
   }
}
